#! /usr/bin/env python

from bf2 import BF2Client
from index import webPage



# BF2142 rcon client
class BF2142Client(BF2Client):
    availMaps = [
        {'mapname': 'verdun'          , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'tunis_harbor'    , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'suez_canal'      , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'sidi_power_plant', 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'shuhia_taiba'    , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'minsk'           , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'fall_of_berlin'  , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'cerbere_landing' , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'camp_gibraltar'  , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']},
        {'mapname': 'belgrade'        , 'mapmodes': ['gpm_cq', 'gpm_ti', 'gpm_coop'], 'mapsizes': ['16', '32', '64']}]

    # return map image location
    def getMapImage(self, map):
        return '/maps/bf2142/%s.jpg' %map['mapname']

    # retrieve player list
    def retrievePlayers(self):
        players = []
        first = True
        result = self.invoke('exec admin.listPlayers')
        for line in result[:-1].split('\n'):
            value = line.split()
            if first and len(value) > 8:
                first = False
                id = int(value[1])
                playername = "%s" %value[3]
                i = 6
                while value[i] != 'ip:':
                    playername = playername + " %s" %value[i-2]
                    i = i + 1
                remoteip = value[i+1]
            elif len(value) > 2:
                first = True
                cdkey = value[2]
                players.append({'id': id,
                                'playername': playername,
                                'remoteip': remoteip,
                                'cdkey': cdkey})
        return players



# BF2142 web page
class BF2142Page(webPage):
    images = webPage.images
    images['logo'] = '/images/bf2142.gif'



if __name__ == '__main__':
    page = BF2142Page(BF2142Client(host='localhost', port=4711, password='Prutskip'))
    page.show()
    del page
