#! /usr/bin/env python

import socket
from time import sleep
from index import rconClient, webPage



# CoD2 rcon client
class CoD2Client(rconClient):
    availMaps = [
        {'mapname': 'mp_breakout'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_brecourt'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_burgundy'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_carentan'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_dawnville'   , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_decoy'       , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_downtown'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_farmhouse'   , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_leningrad'   , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_matmata'     , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_railyard'    , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_toujane'     , 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']},
        {'mapname': 'mp_trainstation', 'mapmodes': ['dm', 'tdm', 'sd', 'ctf', 'hq'], 'mapsizes': ['-']}]

    settingKeys = [
        'scr_teambalance',
        'scr_friendlyfire']

    def __init__(self, host, port, password):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.sock.setblocking(0) # set to non blocking mode
        self.sock.connect((host, port))
        self.password = password

    def __del__(self):
        self.sock.close()

    def invoke(self, cmd):
        packet = '\xFF\xFF\xFF\xFFrcon "%s" %s\n' %(self.password, cmd)
        while len(packet) > 0:
            c = self.sock.send(packet)
            packet = packet[c:]
        sleep(1) # give server time to response to packet
        result = self.getResponse()
        return result

    def getResponse(self):
        result = ""
        done = False
        retries = 0
        while not done and retries <= 50:
            try:
                data = self.sock.recv(1024)
                result = result + data
                p = result.rfind('\xFF\xFF\xFF\xFFprint')
                if p >= 0:
                    result = result[p+10:]
                    done = True
            except:
                sleep(0.1)
            retries = retries + 1
        return result



    # retrieve map list
    def retrieveMaps(self):
        maps = []
        mapmode = self.availMaps[0]['mapmodes'][0]
        result = self.invoke('sv_MapRotation')
        for line in result.split('\n'):
            value = line.split()
            for p in range(len(value)):
                if value[p].strip('"') == 'gametype':
                    mapmode = value[p+1]
                elif value[p] == 'map':
                    mapname = value[p+1].strip('^7"')
                    maps.append({'id': mapname,
                                 'mapname': mapname,
                                 'mapmode': mapmode,
                                 'mapsize': '-'})
        return maps

    # remove maps from map list
    def removeMaps(self, mapIDs):
        maps = self.retrieveMaps()
        i = 0
        while i < len(maps):
            if len(maps) > 1 and maps[i]['id'] in mapIDs:
                maps.remove(maps[i])
            else:
                i = i + 1
        s = ''
        for map in maps: s = s + 'gametype %s map %s ' %(map['mapmode'], map['mapname'])
        self.invoke('sv_MapRotation %s' %s[:-1])

    # move maps to top of map list
    def movetotopMaps(self, mapIDs):
        i = 0
        maps = self.retrieveMaps()
        for map in maps:
            if map['id'] in mapIDs:
                maps.remove(map)
                maps.insert(i, map)
                i = i + 1
        s = ''
        for map in maps: s = s + 'gametype %s map %s ' %(map['mapmode'], map['mapname'])
        self.invoke('sv_MapRotation %s' %s[:-1])

    # append map to map list
    def appendMap(self, mapname, mapmode, mapsize):
        maps = self.retrieveMaps()
        s = ''
        for map in maps: s = s + 'gametype %s map %s ' %(map['mapmode'], map['mapname'])
        s = s + 'gametype %s map %s ' %(mapmode, mapname)
        self.invoke('sv_MapRotation %s' %s[:-1])

    # return current map
    def currentMap(self):
        s = self.invoke('mapname').split()
        try:
            mapname = s[2].strip('^7"')
            map = {'id': mapname,
                   'mapname': mapname,
                   'mapmode': '?',
                   'mapsize': '-'}
        except:
            map = {'id': '?',
                   'mapname': 'unknown',
                   'mapmode': '?',
                   'mapsize': '-'}
        return map

    # return next map
    def nextMap(self):
        currentMap = self.currentMap()
        maps = self.retrieveMaps()
        try:
            i = 0
            while maps[i]['id'] != currentMap['id']: i = i + 1
            map = maps[(i+1)%len(maps)]
        except:
            map = {'id': '?',
                   'mapname': 'unknown',
                   'mapmode': '?',
                   'mapsize': '-'}
        return map

    # restart current map
    def restartMap(self):
        return self.invoke('map_restart')

    # run next map
    def runNextMap(self):
        return self.invoke('map_rotate')

    # set next map
    def setNextMap(self, id):
        return self.invoke('map %s' %id)

    # return map image location
    def getMapImage(self, map):
        return '/maps/cod2/%s.jpg' %map['mapname']



    # retrieve player list
    def retrievePlayers(self):
        players = []
        i = 0
        result = self.invoke('status')
        for line in result.split('\n'):
            i = i + 1
            value = line.split()
            if i >= 4 and len(value) >= 4:
                id = int(value[0])
                playername = ""
                for p in range(4, len(value) - 4):
                    playername = playername + value[p]
                playername = playername.strip('^7')
                remoteip = value[-3]
                cdkey = ''
                players.append({'id': id,
                                'playername': playername,
                                'remoteip': remoteip,
                                'cdkey': cdkey})
        return players

    # retrieve banned players
    def retrieveBannedPlayers(self):
        bannedPlayers = []
        return bannedPlayers

    # kick players
    def kickPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('clientKick %s' %id)

    # ban players
    def banPlayers(self, playerIDs, duration = 'round'):
        for id in playerIDs:
            if duration == 'round':
                self.invoke('tempBanClient %s' %id)
            else:
                self.invoke('BanClient %s' %id)

    # unban players
    def unbanPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('unban %s' %id) #player['playername']) needed ???



    # retrieve server settings
    def retrieveSettings(self):
        settings = {}
        for key in self.settingKeys:
            s = self.invoke(key).split()
            try:
                value = s[2].strip('^7"')
                settings[key] = value
            except:
                pass
        return settings

    # save server settings
    def saveSettings(self, settings):
        for key in settings:
            result = self.invoke('%s %s' %(key, settings[key]))
        return result



    # send message to all players
    def sayAll(self, message):
        result = self.invoke('say "%s"' %message)
        return result



# CoD2 web page
class CoD2Page(webPage):
    images = webPage.images
    images['logo'] = '/images/cod2.gif'



if __name__ == '__main__':
    page = CoD2Page(CoD2Client(host='192.168.1.11', port=28960, password='Prutskip'))
    page.show()
    del page
