#! /usr/bin/env python

import socket
import md5
from time import sleep
from string import lower
from index import rconClient, webPage



# huffman bitcodes
codes    = {
    'a': '010101',
    'A': '000010',
    'b': '101010100',
    'B': '101001000',
    'c': '0101101',
    'C': '100100',
    'd': '0101110',
    'D': '10100001',
    'e': '1001101',
    'E': '101001001',
    'f': '101010101',
    'F': '000011',
    'g': '101010110',
    'G': '001010',
    'h': '101010111',
    'H': '101001010',
    'i': '0101001',
    'I': '01011000',
    'j': '101011000',
    'J': '101001011',
    'k': '101011001',
    'K': '101001100',
    'l': '101011010',
    'L': '0010111',
    'm': '101011011',
    'M': '101001101',
    'n': '1001110',
    'N': '101001110',
    'o': '0001011',
    'O': '10100010',
    'p': '0101111',
    'P': '01111',
    'q': '101011100',
    'Q': '101001111',
    'r': '010010',
    'R': '01110',
    's': '010011',
    'S': '1001100',
    't': '01000',
    'T': '100101',
    'u': '1001111',
    'U': '0101000',
    'v': '101011101',
    'V': '101010000',
    'w': '101011110',
    'W': '101010001',
    'x': '101011111',
    'X': '000100',
    'y': '101100000',
    'Y': '101010010',
    'z': '101100001',
    'Z': '101010011',
    ' ': '111',
    '_': '1011111100',
    '0': '1000',
    '1': '0110',
    '2': '00100',
    '3': '10100000',
    '4': '00011',
    '5': '00000',
    '6': '0001010',
    '7': '010110011',
    '8': '101000110',
    '9': '101000111',
    '$': '1011100110',
    '-': '1011101111',
    "'": '1011101001',
    '.': '00101101',
    ':': '1011110001',
    '=': '1011110100',
    '#': '1011100101',
    '!': '1011100011',
    '>': '1011110101',
    '<': '1011110011',
    '(': '1011101010',
    ')': '1011101011',
    '[': '1011111000',
    ']': '1011111010',
    '{': '1011111110',
    '}': '1100000000',
    '*': '1011101100',
    ',': '1011101110',
    '/': '1011110000',
    '`': '1011111101',
    '"': '1011100100',
    '^1':'1011000100', # ignore bitcode
    '^2':'1011001100'} # ignore bitcode

endOfLine= '0011'
#nextLine = '10100111111111111111111111111011000100'
nextLine = '1010011111111111111111111111'

def encode(cmd):
    s = ''
    for c in cmd: s = s + codes[c]
    s = s + '0011' # add end limiter
    while len(s) % 8 != 0: s = s + '0' # add ending '0' to last byte
    result = ''
    for i in range(0, len(s), 8): result = result + chr(int(s[i:i+8],2))
    return result

def decode(cmd):
    s = ''
    for c in cmd:
        n = ord(c)
        s = s + "".join([str((n >> y) & 1) for y in range(8-1, -1, -1)])
    result = ''
    while len(s) > 0:
        found = False
        for c in codes.keys():
            bits = codes[c]
            if s.startswith(bits):
                if not c.startswith('^'): result = result + c
                s = s[len(bits):]
                found = True
        if s.startswith(endOfLine):
            result = result + '\n'
            # search start of next line
            s = s[len(endOfLine):]
            while len(s) > 0 and not s.startswith(nextLine): s = s[1:]
            s = s[len(nextLine):]
            found = True
        if not found:
            print ">>>undecoded:%s<br>" %s
            break
    return result

# Farcry rcon client
class FarcryClient(rconClient):
    availMaps = [
        {'mapname': 'mp_airstrip'   , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_cliffside'  , 'mapmodes': ['tdm', 'ffa'], 'mapsizes': ['-']},
        {'mapname': 'mp_dam'        , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_dune'       , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_freighter'  , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_jungle'     , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_mangoriver' , 'mapmodes': ['assault']   , 'mapsizes': ['-']},
        {'mapname': 'mp_monkeybay'  , 'mapmodes': ['tdm', 'ffa'], 'mapsizes': ['-']},
        {'mapname': 'mp_monkeybay_n', 'mapmodes': ['tdm', 'ffa'], 'mapsizes': ['-']},
        {'mapname': 'mp_radio'      , 'mapmodes': ['tdm', 'ffa'], 'mapsizes': ['-']},
        {'mapname': 'mp_surf'       , 'mapmodes': ['tdm', 'ffa'], 'mapsizes': ['-']}]

    settingKeys = [
        'gr_respawntime',
        'gr_friendlyfire']

    def __init__(self, host, port, password, mapFile):
        self.host = host
        self.txPort = port
        self.rxPort = 34000
        self.password = password
        self.mapFile = mapFile

    def invoke(self, cmd):
        txSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        txSock.setblocking(0) # set to non blocking mode
        txSock.bind(('0.0.0.0', self.rxPort))
        txSock.connect((self.host, self.txPort))
        m = md5.new(self.password)
        packet = '\x09\xff\xff\xff'+m.digest()+encode(cmd)
        while len(packet) > 0:
            c = txSock.send(packet)
            packet = packet[c:]
        txSock.close()
        result = self.getResponse()
        #debug print ">>>invoke(%s)=%s<br>" %(cmd,result.split('\n'))
        return result

    def getResponse(self):
        rxSock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        rxSock.setblocking(0) # set to non blocking mode
        rxSock.bind(('', self.rxPort))
        result = ""
        done = False
        retries = 0
        while not done and retries <= 50:
            try:
                data = rxSock.recv(1024)
                result = result + data
            except:
                if result:
                    done = True
                else:
                    sleep(0.1)
                    retries = retries + 1
        rxSock.close()
        result = decode(result[4:]) # remove first 4 bytes header
        return result



    #retrieve map list
    def retrieveMaps(self):
        maps = []
        try:
            f = file(self.mapFile, 'r')
            for line in f.readlines():
                value = line.split()
                if len(value) == 2:
                    mapname = lower(value[0])
                    mapmode = lower(value[1])
                    maps.append({'id': mapname,
                                 'mapname': mapname,
                                 'mapmode': mapmode,
                                 'mapsize': '-'})
            f.close()
        except:
            pass
        return maps

    def reloadMaps(self):
        return None

    def saveMaps(self):
        return None

    def removeMaps(self, mapIDs):
        maps = self.retrieveMaps()
        i = 0
        while i < len(maps):
            if len(maps) > 1 and maps[i]['id'] in mapIDs:
                maps.remove(maps[i])
            else:
                i = i + 1
        try:
            f = file(self.mapFile, 'w')
            for map in maps:
                f.write('%s %s\n' %(map['mapname'], map['mapmode']))
            f.close()
            self.envoke('sv_reloadmapcycle')
        except:
            pass

    def movetotopMaps(self, mapIDs):
        i = 0
        maps = self.retrieveMaps()
        for map in maps:
            if map['id'] in mapIDs:
                maps.remove(map)
                maps.insert(i, map)
                i = i + 1
        try:
            f = file(self.mapFile, 'w')
            for map in maps:
                f.write('%s %s\n' %(map['mapname'], map['mapmode']))
            f.close()
            self.envoke('sv_reloadmapcycle')
        except:
            pass

    def appendMap(self, mapname, mapmode, mapsize):
        try:
            f = file(self.mapFile, 'a')
            f.write('%s %s\n' %(mapname, mapmode))
            f.close()
            self.envoke('sv_reloadmapcycle')
        except:
            pass

    def currentMap(self):
        result = self.invoke('g_levelname')
        try:
            lines = result.split('\n')
            value = lines[1].split('=')[1]
            mapname = value.strip(' ')
            map = {'id': mapname,
                   'mapname': mapname,
                   'mapmode': '?',
                   'mapsize': '-'}
        except:
            map = {'id': '?',
                   'mapname': 'unknown',
                   'mapmode': '?',
                   'mapsize': '-'}
        return map

    def nextMap(self):
        result = self.invoke('gr_nextmap')
        try:
            lines = result.split('\n')
            value = lines[1].split('=')[1]
            mapname = value.strip(' ')
            map = {'id': mapname,
                   'mapname': mapname,
                   'mapmode': '?',
                   'mapsize': '-'}
        except:
            map = {'id': '?',
                   'mapname': 'unknown',
                   'mapmode': '?',
                   'mapsize': '-'}
        return map

    # restart current map
    def restartMap(self):
        return self.invoke('sv_restart')

    # start next map
    def runNextMap(self):
        result = ''
        maps = self.retrieveMaps()
        nextMap = self.nextMap()
        for map in maps:
            if map['id'] == nextMap['id']:
                result = self.invoke('sv_changemap %s %s' %(map['mapname'], map['mapmode']))
                break
        return result

    # set next map
    def setNextMap(self, id):
        return self.invoke('gr_nextmap %s' %id)

    # return map image location
    def getMapImage(self, map):
        return '/maps/farcry/%s.jpg' %map['mapname']



    # retrieve player list
    def retrievePlayers(self):
        players = []
        i = 0
        result = self.invoke('listplayers')
        for line in result.split('\n'):
            i = i + 1
            value = line.split()
            if i >= 2 and len(value) >= 2:
                id = value[0]
                playername = value[1]
                remoteip = ''
                cdkey = ''
                players.append({'id': id,
                                'playername': playername,
                                'remoteip': remoteip,
                                'cdkey': cdkey})
        return players

    # retrieve banned player list
    def retrieveBannedPlayers(self):
        bannedPlayers = []
        i = 0
        result = self.invoke('listban')
        for line in result.split('\n'):
            i = i + 1
            value = line.split()
            if i >= 3 and len(value) >= 2:
                id = value[0]
                playername = value[1]
                bannedPlayers.append({'id': id,
                                      'playername': playername,
                                      'duration': '-'})
        return bannedPlayers

    def kickPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('kickid %s' %id)

    def banPlayers(self, playerIDs, duration = 'round'):
        for id in playerIDs:
            self.invoke('banid %s' %id)

    def unbanPlayers(self, playerIDs):
        for id in playerIDs:
            self.invoke('unban %s' %id)

    def clearBanList(self):
        bannedPlayers = self.retrieveBannedPlayers()
        for player in bannedPlayers:
            self.unbanPlayers(player['id'])



    def retrieveSettings(self):
        settings = {}
        for key in self.settingKeys:
            result = self.invoke(key)
            for line in result.split('\n'):
                value = line.split('=')
                if len(value) >= 2:
                    settings[key] = value[1]
        return settings

    def saveSettings(self, settings):
        for key in settings:
            result = self.invoke('%s %s' %(key, settings[key]))
        return result



    # send message to all players
    def sayAll(self, message):
        result = self.invoke('say "%s"' %message)
        return result



# Farcry web page
class FarcryPage(webPage):
    images = webPage.images
    images['logo'] = '/images/farcry.gif'



if __name__ == '__main__':
    page = FarcryPage(FarcryClient(host='192.168.1.11', port=49001, password='Prutskip',
        mapFile ='d:\\downloads\\far Cry\\profiles\\server\\mapcycle.txt'))
    page.show()
    del page
